pub struct LiveVariableAnalysis {
pub live_at_entry: HashMap<u32, BitVec<u32>>,
pub live_at_exit: HashMap<u32, BitVec<u32>>,
}Expand description
Live variable analysis results.
Memory Optimization
- Live variable sets use
BitVec<u32>instead ofHashSet<u8>- Saves memory: 1 bit per register (32 bits = 4 bytes) vs 8 bytes per register + overhead
- Faster membership tests: O(1) bit access with better cache locality
- Efficient set operations: bitwise AND/OR for intersection/union
Algorithm
Uses iterative data flow analysis:
- Initialize all blocks with empty live sets
- Iterate until fixed point:
- Live at exit = union of live at entry of successors
- Live at entry = (Live at exit - Killed) ∪ Generated
Fields
live_at_entry: HashMap<u32, BitVec<u32>>Live variables at entry of each basic block Key: block ID (u32), Value: bit vector of live registers (1 bit per register)
live_at_exit: HashMap<u32, BitVec<u32>>Live variables at exit of each basic block Key: block ID (u32), Value: bit vector of live registers (1 bit per register)